from co6co.enums import Base_Enum, Base_EC_Enum


class resource_category(Base_Enum):
    """
    资源类型
    """
    image = "image", 0
    video = "video", 1
    file = "file", 2


class dict_state(Base_EC_Enum):
    """
    字典和字典类型使用的状态
    """
    enabled = "enabled", "启用", 1
    disabled = "disabled", "禁用", 0


class menu_type(Base_EC_Enum):
    """
    菜单类型 
    """
    group = "group", "分组", 0  # 分组菜单
    api = "api", "API接口", 1   # api
    view = "view", "页面视图", 2  # a视图
    subView = "subView", "页面子视图", 3  # a视图
    button = "button", "视图功能", 10  # 视图中的按钮等。


class menu_state(Base_EC_Enum):
    """
    菜单类型 
    """

    enabled = "enabled", "启用", 0
    disabled = "disabled", "禁用", 1


class user_category(Base_EC_Enum):
    normal = "normal", "普通", 0
    system = "system", "系统", 1
    terminal = "terminal ", "终端", 2


class user_state(Base_EC_Enum):
    enabled = "enabled", "启用", 0
    disabled = "disabled", "禁用", 1
    locked = "locked", "锁定", 2



from __future__ import annotations
from co6co_db_ext.po import BasePO, TimeStampedModelPO, UserTimeStampedModelPO, CreateUserStampedModelPO
from sqlalchemy import func, INTEGER, SmallInteger, Integer, UUID,  INTEGER, BigInteger, Column, ForeignKey, String, DateTime, CheckConstraint
from sqlalchemy.orm import relationship, declarative_base, Relationship
import co6co.utils as tool
from co6co.utils import hash
import sqlalchemy
from sqlalchemy.schema import DDL
from sqlalchemy import MetaData
import uuid


class sysConfigPO(UserTimeStampedModelPO):
    """
    系统配置
    """
    __tablename__ = "sys_config"
    id = Column("id", Integer, autoincrement=True, primary_key=True)
    name = Column("name", String(64))
    code = Column("code", String(64),  unique=True)
    sysFlag = Column("sys_flag", String(1),  comment="Y:系统,N:不是")
    dictFlag = Column("dict_flag", String(1),  comment="Y:使用字典做配置,N:手动配置")
    dictTypeId = Column("dict_type_id", Integer,  comment="字典类型ID")
    value = Column("value", String(2048),  comment="配置值")
    remark = Column("remark", String(2048), comment="备注")

    def update(self, po: sysConfigPO):
        self.name = po.name
        self.code = po.code
        self.dictFlag = po.dictFlag
        self.sysFlag = po.sysFlag
        self.value = po.value
        self.remark = po.remark


class sysDictTypePO(UserTimeStampedModelPO):
    """
    字典类型
    """
    __tablename__ = "sys_dict_type"
    id = Column("id", Integer, autoincrement=True, primary_key=True)
    name = Column("name", String(64))
    code = Column("code", String(64),  unique=True)
    desc = Column("desc", String(1024))
    sysFlag = Column("sys_flag", String(1), comment="系统标识:Y/N")
    # py 层限制 取值范围为:(0-1)
    state = Column("state", SmallInteger,  CheckConstraint(
        'state >= 0 AND state <= 1'), comment="状态:0/1->禁用/启用",)
    order = Column("order", Integer, comment="排序")

    def update(self, po: sysDictTypePO):
        self.name = po.name
        self.code = po.code
        self.desc = po.desc
        self.sysFlag = po.sysFlag
        self.state = po.state
        self.order = po.order


class sysDictPO(UserTimeStampedModelPO):
    """
    字典
    """
    __tablename__ = "sys_dict"
    id = Column("id", Integer, autoincrement=True, primary_key=True)
    dictTypeId = Column("dict_type_id", Integer, comment="字典类型ID")
    name = Column("name", String(64))
    flag = Column("flag", String(64))
    value = Column("value", String(1024))
    desc = Column("desc", String(1024))
    # py 层限制 取值范围为:(0-1)
    state = Column("state", SmallInteger,  CheckConstraint(
        'state >= 0 AND state <= 1'), comment="状态:0/1->禁用/启用",)
    order = Column("order", Integer, comment="排序")

    def update(self, po: sysDictPO):
        self.dictTypeId = po.dictTypeId
        self.name = po.name
        self.flag = po.flag
        self.value = po.value
        self.desc = po.desc

        self.state = po.state
        self.order = po.order




from .bll import BaseBll
from co6co_permissions.model.pos.other import sysConfigPO
from sqlalchemy import Select
from co6co_db_ext.db_utils import QueryOneCallable
import json as sysJson
from co6co.utils import log


class config_bll(BaseBll):
    async def query_config_value(self,  key: str, parseDict: bool = False) -> str | dict:
        select = (
            Select(sysConfigPO.value)
            .filter(sysConfigPO.code.__eq__(key))
        )
        call = QueryOneCallable(self.session)
        result = await call(select, isPO=False)
        if not result:
            log.warn("未找到'{}'的相关配置".format(key))
            return {} if parseDict else None
        result: str = result.get("value")
        if parseDict:
            result = sysJson.loads(result)
        return result



from co6co.utils import log
import os
import datetime
import cv2
from cv2.typing import MatLike
from websockets.sync.client import connect
import numpy as np
from co6co.utils.File import File


def showImage(wsUrl: str):
    first = None
    t: int = 100
    with connect(wsUrl) as ws:
        all: bytes = None
        while t > 0:
            # 接收消息
            result = ws.recv()
            if all == None:
                all = result
            else:
                all = all+result
            # 解析消息为图像数据

            # img_data = np.fromstring(result, dtype='uint8')
            # img_data = np.asarray( bytearray( result[3:]), dtype="uint8")
            # img = base64.b64decode(all)
            img = cv2.imdecode(np.frombuffer(
                all, dtype='uint8'), cv2.IMREAD_COLOR)
            if img == None:
                print("is None")
            else:  # 显示图像
                cv2.imshow('WebSocket Image', img)
                break
            cv2.waitKey(1)
    ws.close()


def getVideoFragment(wsUrl: str):
    try:
        with connect(wsUrl) as ws:
            # 接收消息
            result = ws.recv()
            s = getTempFileName("t.mp4")
            File.writeFile(s, result)
            return s
    except Exception as e:
        return None


def getTempFileName(ext: str = "jpg"):
    if not os.path.exists("tmp"):
        os.makedirs("tmp")
    s = f"tmp/frame_{datetime.datetime.now().strftime('%H%M%S%f')}.{ext}"
    return s


class VideoCamera(object):
    """
    摄像头
    """
    video: cv2.VideoCapture

    def __init__(self):
        self.video = cv2.VideoCapture(0)

    def __del__(self):
        self.video.release()

    def get_frame(self):
        success, image = self.video.read()
        ret, jpeg = cv2.imencode('.jpg', image)
        return jpeg.tobytes()


def resize_image(imreadImage: MatLike, height: int = 208, width: int = 117) -> MatLike:
    """
    按指定图像大小调整尺寸
    """
    top, bottom, left, right = (0, 0, 0, 0)
    # 获取图片尺寸
    h, w, _ = imreadImage.shape

    # 对于长宽不等的图片，找到最长的一边
    longest_edge = max(h, w)

    # 计算短边需要增加多少像素宽度才能与长边等长(相当于padding，长边的padding为0，短边才会有padding)
    if h < longest_edge:
        dh = longest_edge - h
        top = dh // 2
        bottom = dh - top
    elif w < longest_edge:
        dw = longest_edge - w
        left = dw // 2
        right = dw - left
    else:
        pass  # pass是空语句，是为了保持程序结构的完整性。pass不做任何事情，一般用做占位语句。

    # RGB颜色
    BLACK = [0, 0, 0]
    # 给图片增加padding，使图片长、宽相等
    # top, bottom, left, right分别是各个边界的宽度，cv2.BORDER_CONSTANT是一种border type，表示用相同的颜色填充
    constant = cv2.copyMakeBorder(
        imreadImage, top, bottom, left, right, cv2.BORDER_CONSTANT, value=BLACK)
    # 调整图像大小并返回图像，目的是减少计算量和内存占用，提升训练速度
    return cv2.resize(constant, (height, width))


async def screenshot(videoPathOrStreamUrl: str, w: int = 208, h: int = 117, isFile: bool = True, useBytes: bool = False) -> str | bytes | None:
    """
    视频截图
    视频第一帧作为 poster
    """

    if (isFile and os.path.exists(videoPathOrStreamUrl)) or videoPathOrStreamUrl:
        try:
            userVideoFragment = False
            if 'wss://' in videoPathOrStreamUrl or 'wss://' in videoPathOrStreamUrl:
                videoPathOrStreamUrl = getVideoFragment(videoPathOrStreamUrl)
                userVideoFragment = True
            cap = cv2.VideoCapture(videoPathOrStreamUrl,
                                   cv2.CAP_FFMPEG)  # 打开视频
            if not cap.isOpened():
                log.warn(f"未能打开：{videoPathOrStreamUrl}")
                return None

            '''
            # 视频信息
            fps = int(cap.get(cv2.CAP_PROP_FPS))
            width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
            height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
            # 创建输出视频文件对象，参数为输出文件名、编解码器、帧率、宽度和高度信息
            out = cv2.VideoWriter('output.avi', cv2.VideoWriter_fourcc(*'XVID'), fps, (width, height))

            cap.set(cv2.CAP_PROP_FPS, 30)
            cap.set(cv2.CAP_PROP_FRAME_WIDTH, 1920)
            cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 1080) 
            '''
            # cap.set(cv2.CAP_PROP_READ_TIMEOUT_MSEC , 5)
            ret, fram = cap.read()
            if not ret:
                return None
            if useBytes:
                fram = resize_image(fram, w, h)
                ret, jpeg = cv2.imencode('.jpg', fram)
                return jpeg.tobytes()
            s = None
            if type(fram) == np.ndarray:
                s = getTempFileName()
                fram = resize_image(fram, w, h)
                cv2.imwrite(s, fram)
                return s
        finally:
            cap.release()
            if userVideoFragment:
                os.remove(videoPathOrStreamUrl)
    return None



# -*- coding:utf-8 -*-
'''
import datetime
now = datetime.datetime.now()

month = now.month
day = now.day
hour = now.hour
minute = now.minute
# f"{number:.2f}"
# f"{number:02d}"
s = f"{month:02d}{day:02d}{hour:02d}{minute:02d}"

__version_info = (0, 0, 3, int(s))
'''
__version_info = (0, 0, 28)
__version__ = ".".join([str(x) for x in __version_info])



from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete
from co6co_db_ext.db_utils import db_tools
from co6co_web_db.model.params import associationParam


from .base_view import AuthMethodView
from ..model.pos.right import UserPO, RolePO, UserRolePO, AccountPO
from ..model.filters.user_filter import user_filter


class changePwd_view(AuthMethodView):
    routePath = "/changePwd"

    async def post(self, request: Request):
        """
        修改密码：
        {
               oldPassword:""
               newPassword:""
        }
        """
        data = request.json

        userId = self.getUserId(request)
        userName = self.getUserName(request)

        oldPassword = data["oldPassword"]
        password = data["newPassword"]
        select = (Select(UserPO).filter(UserPO.userName == userName))

        async def edit(_, one: UserPO):
            if one != None:
                if one.password != one.encrypt(oldPassword):
                    return JSON_util.response(Result.fail(message="输入的旧密码不正确！"))
                if one.encrypt(password) == one.encrypt(oldPassword):
                    return JSON_util.response(Result.fail(message="输入的旧密码与新密码一样！"))

                one.password = one.encrypt(password)
            return JSON_util.response(Result.success())
        return await self.update_one(request, select, edit)


class user_info_view(AuthMethodView):
    routePath = "/currentUser"

    async def get(self, request: Request):
        """
        当前用户信息  
        return {
            data:{
                avatar:""
                remark:""
            } 
        }
        """
        userName = self.getUserName(request)
        select = Select(UserPO.avatar, UserPO.remark).filter(UserPO.userName == userName)
        dict = self.get_one(request, select, isPO=False)
        return JSON_util.response(Result.success(data=dict))



from sanic import Sanic, Blueprint,Request
from co6co_sanic_ext .api import add_routes
from ..view_model.role_view import role_view,roles_view,roles_ass_view

role_api = Blueprint("roles_API",url_prefix="/role") 
add_routes(role_api,role_view,roles_view,roles_ass_view)

'''
userGroup_api.add_route(roles_view.as_view(),"/",name=roles_view.__name__)  
userGroup_api.add_route(role_view.as_view(),"/<pk:int>",name=role_view.__name__) 
'''


from sanic.request import Request


from sqlalchemy .orm.attributes import InstrumentedAttribute
from typing import Tuple
from co6co_db_ext.db_filter import absFilterItems
from co6co.utils import log
from sqlalchemy import func, or_, and_, Select
from ..pos.other import sysDictTypePO
from ...view_model.aop.authonCache import AuthonCacheManage


class Filter(absFilterItems):
    """
    字典类型 filter
    """
    name: str = None
    code: str = None

    def __init__(self):
        super().__init__(sysDictTypePO)

    def filter(self) -> list:
        """
        过滤条件
        """
        filters_arr = []

        if self.checkFieldValue(self.name):
            filters_arr.append(sysDictTypePO.name.like(f"%{self.name}%"))
        if self.checkFieldValue(self.code):
            filters_arr.append(sysDictTypePO.code.like(f"%{self.code}%"))

        return filters_arr

    def getDefaultOrderBy(self) -> Tuple[InstrumentedAttribute]:
        """
        默认排序
        """
        return (sysDictTypePO.order.asc(),)



from sanic import Sanic, Blueprint, Request
from co6co_sanic_ext .api import add_routes
from ..view_model.user import user_ass_view, users_view, user_view, sys_users_view, ticketView
from ..view_model.currentUser import changePwd_view, user_info_view
from ..view_model.login import login_view
from ..view_model.aop.api_auth import authorized
from ..model.enum import user_state, user_category
from co6co_db_ext.db_utils import db_tools
from ..model.pos.right import UserPO
from co6co_sanic_ext.model.res.result import Result
from co6co_sanic_ext.utils import JSON_util
from ..view_model.aop import exist

user_api = Blueprint("users_API", url_prefix="/users")
add_routes(user_api, login_view, ticketView)
add_routes(user_api, changePwd_view, user_info_view)
add_routes(user_api, user_ass_view, users_view, user_view, sys_users_view)


@user_api.route("/status", methods=["GET", "POST"])
@authorized
async def getUserStatus(request: Request):
    """
    用户状态
    """
    states = user_state.to_dict_list()
    return JSON_util.response(Result.success(data=states))


@user_api.route("/category", methods=["GET", "POST"])
@authorized
async def getUserCategory(request: Request):
    """
    用户类别
    """
    states = user_category.to_dict_list()
    return JSON_util.response(Result.success(data=states))


@user_api.route("/exist/<userName:str>/<pk:int>", methods=["GET"])
@authorized
async def userExist(request: Request, userName: str, pk: int = 0):
    """
    用户名是否存在
    """
    result = await db_tools.exist(request.ctx.session, UserPO.userName == userName, UserPO.id != pk)
    return exist(result, "用户", userName)


@user_api.route("/exist", methods=["POST"])
@authorized
async def userExistPost(request: Request):
    """
    用户名是否存在
    """
    id = request.json.get("id")
    userName = request.json.get("userName")
    result = await db_tools.exist(request.ctx.session, UserPO.userName == userName, UserPO.id != id)
    return exist(result, "用户", userName)



from sanic import Sanic, Blueprint, Request
from co6co_sanic_ext.api import add_routes
from ..view_model.verify.drapVerify import drap_verify_view
from ..view_model.user_group import user_groups_sub_tree_view


verify_api = Blueprint("verify_api", url_prefix="/verify")
add_routes(verify_api, drap_verify_view)



import asyncio
from sqlalchemy.ext.asyncio import AsyncSession
from co6co_db_ext.db_session import db_service
import asyncio
from sanic import Sanic
from co6co.task.thread import ThreadEvent
from co6co.utils import log
from sqlalchemy import text


class baseBll:
    session: AsyncSession = None
 
    def __init__(self, db_settings: dict) -> None:
        _service: db_service = db_service(db_settings)
        self.session: AsyncSession = _service.async_session_factory() 
        self.service=_service
        self.isClose=False
        '''
        service:db_service=app.ctx.service
        self.session:AsyncSession=service.async_session_factory()
        '''
        # log.warn(f"..创建session。。")
        pass
    async def close(self):
        #log.log("等待关闭。。。",type(self.session))
        self.isClose=True 
        await self.session.close() 
        await self.service.engine.dispose()
        #log.log("关闭ed")
        pass
    def __del__(self) -> None: 
        if not self.isClose :
            loop= asyncio.get_running_loop() 
            loop.create_task(self.close())  
            #log.succ(f"会在close之前执行 {self}...关闭session",'done->',task.done(),"canceled->",task.cancelled())
             

    def __repr__(self) -> str:
        return f'{self.__class__}'


class BaseBll(baseBll): 
    def __init__(self,*,  db_settings: dict={},app:Sanic=None) -> None: 
        self.t = ThreadEvent() 
        if not db_settings:
            app =app or Sanic.get_app()
            db_settings=app.config.db_settings 
        super().__init__(db_settings)

    def run(self, task, *args, **argkv):
        data = self.t.runTask(task, *args, **argkv)
        return data
    def __str__(self):
        return f'{self.__class__}'

    def __del__(self) -> None:
        try: 
            if not self.isClose: 
                self.t.runTask(self.close) 
            self.t.close()  
        except Exception as e:
            log.warn("__del___ error",e)
            pass




from ..pos.right import UserPO, UserGroupPO
from sqlalchemy .orm.attributes import InstrumentedAttribute
from typing import Tuple
from co6co_db_ext.db_filter import absFilterItems
from co6co.utils import log
from sqlalchemy import func, or_, and_, Select


class user_filter(absFilterItems):
    """
        用户表过滤器
    """
    name: str = None
    userGroupId: int = None
    state: int = None

    def __init__(self, userName=None, userGroupId: int = None):
        super().__init__(UserPO)
        self.name = userName
        self.userGroupId = userGroupId
        self.listSelectFields = [UserPO.id, UserPO.userGroupId, UserPO.category, UserPO.state, UserPO.createTime, UserPO.userName]

    def filter(self) -> list:
        """
        过滤条件
        """
        filters_arr = []
        if self.checkFieldValue(self.userGroupId):
            filters_arr.append(UserPO.userGroupId.__eq__(self.userGroupId))
        if self.checkFieldValue(self.state):
            filters_arr.append(UserPO.state.__eq__(self.state))
        if self.checkFieldValue(self.name):
            filters_arr.append(UserPO.userName.like(f"%{self.name}%"))
        return filters_arr

    def create_List_select(self):
        select = (
            Select(UserPO.id, UserPO.userGroupId, UserPO.state, UserPO.category, UserPO.createTime, UserPO.userName, UserGroupPO.name.label("groupName"), UserGroupPO.id.label("groupId"))
            .join(UserGroupPO, isouter=True, onclause=UserPO.userGroupId == UserGroupPO.id)
            .filter(and_(*self.filter()))
        )
        return select

    def getDefaultOrderBy(self) -> Tuple[InstrumentedAttribute]:
        """
        默认排序
        """
        return (UserPO.createTime.desc(),)




from co6co_web_db.view_model import BaseMethodView

from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete
from co6co_db_ext.db_utils import db_tools
from co6co_web_db.services.jwt_service import createToken, decodeToken
from co6co.utils import log

from co6co_web_db.view_model import get_one
from ..model.pos.right import UserPO, RolePO, UserRolePO, AccountPO
from co6co.utils.tool_util import to_underscore
import uuid


def getSecret(request: Result):
    """
    获取加密密钥
    """
    return request.app.config.SECRET


def getCurrentUserId(request: Request):
    if "current_user" in request.ctx.__dict__.keys():
        return request.ctx.current_user["id"]
    return None


def getCurrentUserName(request: Request):
    if "current_user" in request.ctx.__dict__.keys():
        return request.ctx.current_user["userName"]
    return None


def getCtxData(user: UserPO):
    """
    通过user获取 dict 保存在 request.ctx.current_user 中 
    """
    return user.to_jwt_dict()


async def generatePageToken(SECRET: str, user: UserPO, expire_seconds: int = 86400, **kvarg):
    """
    生成登录token和刷新token
    """
    tokenData = await generateUserToken(SECRET, getCtxData(user), expire_seconds=expire_seconds, **kvarg)
    refreshToken = await generateRefreshToken(SECRET, user.id)
    tokenData.update({"refreshToken": refreshToken})
    return tokenData


async def queryUer(session: AsyncSession, userId: int):
    select = Select(UserPO).filter(UserPO.id.__eq__(userId))
    user: UserPO = await db_tools.execForPo(session, select, remove_db_instance_state=False)
    return user


async def generateUserToken(SECRET: str,   data: dict, userOpenId: str = None, expire_seconds: int = 86400):
    """
    SECRET:密钥 
    data:   放置token所需数据 
    userOpenId: role 放置
    expire_seconds: 过期时间,前端使用
    """
    if data != None:
        token = await createToken(SECRET, data, expire_seconds)
        return {"token": token, "expireSeconds": expire_seconds,   "role": userOpenId}
    else:
        raise Exception("data can't is NUll! ")


async def generateRefreshToken(SECRET: str, userId: any,  expire_seconds=30*86400):
    """
    创建刷新Token
    若要使已生成的 刷新 Token 失效可以 修改加密密码
    """
    token = await generateCode(SECRET, userId,  expire_seconds)
    return {"token": token, "expireSeconds": expire_seconds}


async def generateCode(SECRET: str, userId: any,  expire_seconds=180):
    """
    创建 code
    可以通过 code 得到授权token
    """
    token = await createToken(SECRET, {"userId": userId}, expire_seconds)
    return token


async def decodeCode(SECRET: str, codeToken: str):
    """
    创建 code
    可以通过 code 得到授权token
    """
    try:
        data = decodeToken(codeToken, SECRET)
        if "userId" in data:
            return data.get("userId")
        return None
    except:
        return None



from sanic.request import Request
from typing import List, Dict
from co6co.utils import log
from .authonCache import AuthonCacheManage


class apiPermissionCheck:
    request: Request = None
    currentUserMenus: List[Dict] = None
    inited: bool = False

    def __init__(self, request: Request) -> None:
        self.request = request
        pass
    # 协调函数

    async def init(self):
        """
        初始化
        """
        cacheManage = AuthonCacheManage(self.request)
        allMenuData = await cacheManage.menuData
        currentUserRoles = await cacheManage.currentRoles
        self.currentUserMenus = []
        [self.currentUserMenus.append(m) for m in allMenuData if m.get("roleId") in currentUserRoles and m.get("id") not in map(lambda m: m['id'], self.currentUserMenus)]
        self.inited = True
    # def __await__(self):
        # 需要生成器对对象
        # allMenuData= yield from  cacheManage.menuData

    def check(self) -> bool:
        if not self.inited:
            log.err("未初始化.")
            return False
        for menu in self.currentUserMenus:
            if self._check(menu):
                return True
        return False

    def _check(self, menu: Dict):
        url: str = menu["url"]
        path = self.request.path
        method = self.request.method
        methods: str = menu["methods"]
        methods: list = methods.split(",")
        if method not in methods and "ALL" not in methods:
            return False
        pathArr = path.split("/")
        if "**" in url:
            url = url[0:url.index("**")]
            urlArr = url.split("/")
            # log.warn(pathArr,urlArr)
            if len(pathArr) >= len(urlArr)-1 and pathArr[0:len(urlArr)-1] == urlArr[0: len(urlArr)-1]:
                return True
        if "*" in url:
            url = url[0:url.index("*")]
            urlArr = url.split("/")
            log.warn(pathArr, urlArr)
            if (len(pathArr) == len(urlArr) or len(pathArr) == len(urlArr)-1) and pathArr[0:len(urlArr)-1] == urlArr[0: len(urlArr)-1]:
                return True
        if url == path:
            return True
        return False




from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select
from co6co_db_ext.db_utils import db_tools
from .base_view import AuthMethodView
from .aop import exist, ObjectExistRoute
from ..model.pos.right import menuPO
from ..model.filters.menu_filter import menu_filter
from .aop.api_auth import menuChanged


class roles_ass_exist_view(AuthMethodView):
    routePath = ObjectExistRoute

    async def get(self, request: Request, code: str, pk: int = 0):
        result = await self.exist(request, menuPO.code == code,
                                  menuPO.id != pk, column=menuPO.id)
        return exist(result, "菜单编码", code)


class menu_tree_view(AuthMethodView):
    routePath = "/tree"

    async def get(self, request: Request):
        """
        树形选择下拉框数据
        selectTree :  el-Tree
        """
        select = (
            Select(menuPO.id, menuPO.name, menuPO.code, menuPO.parentId)
            .order_by(menuPO.order.asc())
        )
        return await self.query_tree(request, select, pid_field='parentId', id_field="id", isPO=False)

    async def post(self, request: Request):
        """
        树形 table数据
        没有条件 返回   tree_data
        有条件 返回     PAGED_list 

        """
        param = menu_filter()
        param.__dict__.update(request.json)
        if len(param.filter()) > 0:
            return await self.query_page(request, param)
        return await self.query_tree(request, param.create_List_select().order_by(menuPO.order.asc()), rootValue=0, pid_field='parentId', id_field="id")


class menus_view(AuthMethodView):
    async def get(self, request: Request):
        """
        树形选择下拉框数据
        selectTree :  el-Tree
        """
        select = (
            Select(menuPO.id, menuPO.name, menuPO.code, menuPO.parentId)
            .order_by(menuPO.parentId.asc())
        )
        return await self.query_list(request, select,  isPO=False)

    async def post(self, request: Request):
        """
        树形 table数据
        tree 形状 table
        """
        param = menu_filter()
        param.__dict__.update(request.json)

        return await self.query_list(request, param.list_select)

    @menuChanged
    async def put(self, request: Request):
        """
        增加
        """
        po = menuPO()
        userId = self.getUserId(request)
        po.__dict__.update(request.json)
        if type(po.methods) == list:
            po.methods = ",".join(po.methods)

        async def before(po: menuPO, session: AsyncSession, request):
            exist = await db_tools.exist(session,  menuPO.code.__eq__(po.code), column=menuPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))

        return await self.add(request, po, json2Po=False, userId=userId, beforeFun=before)

    def patch(self, request: Request):
        return text("I am patch method")


class menu_view(AuthMethodView):
    routePath = "/<pk:int>"

    @menuChanged
    async def put(self, request: Request, pk: int):
        """
        编辑
        """
        po = menuPO()
        po.__dict__.update(request.json)
        if type(po.methods) == list:
            po.methods = ",".join(po.methods)

        async def before(oldPo: menuPO, po: menuPO, session: AsyncSession, request):
            exist = await db_tools.exist(session, menuPO.id != oldPo.id, menuPO.code.__eq__(po.code), column=menuPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))
            if po.parentId == oldPo.id:
                return JSON_util.response(Result.fail(message=f"'父节点选择错误！"))
        return await self.edit(request, pk, menuPO, po=po, userId=self.getUserId(request), fun=before)

    @menuChanged
    async def delete(self, request: Request, pk: int):
        """
        删除
        """
        async def before(po: menuPO, session: AsyncSession):
            count = await db_tools.count(session, menuPO.parentId == po.id, column=menuPO.id)
            if count > 0:
                return JSON_util.response(Result.fail(message=f"该'{po.name}'节点下有‘{count}’节点，不能删除！"))
        return await self.remove(request, pk, menuPO, beforeFun=before)


class menu_batch_view(AuthMethodView):
    routePath = "/batch"

    @menuChanged
    async def post(self, request: Request):
        """
        批量增加
        """
        data = request.json

        async def before(po: menuPO, session: AsyncSession, request):
            exist = await db_tools.exist(session,  menuPO.code.__eq__(po.code), column=menuPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))
        if isinstance(data, list):
            polist = []
            userId = self.getUserId(request)
            for js in data:
                po = menuPO()
                po.__dict__.update(js)
                polist.append(po)
            return await self.batchAdd(request, polist,   userId=userId, beforeFun=before)
        return self.response_json(Result.fail(message="json 数据不是列表"))




from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete

from co6co_db_ext.db_utils import db_tools, DbCallable
from co6co_web_db.model.params import associationParam

from datetime import datetime
from ..base_view import AuthMethodView
from ..aop import exist
from ...model.filters.dict_type_filter import Filter
from ...model.filters.dict_filter import DictFilter

from ...model.enum import dict_state
from ...model.pos.other import sysDictTypePO, sysDictPO


class DictTypeExistView(AuthMethodView):
    routePath = "/type/exist/<code:str>/<pk:int>"

    async def get(self, request: Request, code: str, pk: int = 0):
        result = await db_tools.exist(request.ctx.session, sysDictTypePO.code == code, sysDictTypePO.id != pk)
        return exist(result, "字典类型", code)


class DictTypeViews(AuthMethodView):
    routePath = "/type"

    async def get(self, request: Request):
        """
        字典类型 下拉 
        """
        select = (
            Select(sysDictTypePO.id, sysDictTypePO.name, sysDictTypePO.code)
            .filter(sysDictTypePO.state.__eq__(dict_state.enabled.val))
            .order_by(sysDictTypePO.order.asc())
        )
        return await self.query_list(request, select,  isPO=False)

    async def post(self, request: Request):
        """
        table数据
        """
        param = Filter()
        return await self.query_page(request, param)

    async def put(self, request: Request):
        """
        增加
        """
        po = sysDictTypePO()
        userId = self.getUserId(request)

        async def before(po: sysDictTypePO, session: AsyncSession, request):
            exist = await db_tools.exist(session,  sysDictTypePO.code.__eq__(po.code), column=sysDictTypePO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))
        return await self.add(request, po, userId=userId, beforeFun=before)


class DictTypeView(AuthMethodView):
    routePath = "/type/<pk:int>"

    async def get(self, request: Request, pk: int):
        """
        获取字典选择
        """
        select = (
            Select(sysDictPO.id, sysDictPO.name,
                   sysDictPO.value, sysDictPO.desc)
            .filter(sysDictPO.dictTypeId.__eq__(pk), sysDictPO.state.__eq__(dict_state.enabled.val))
            .order_by(sysDictPO.order.asc())
        )
        return await self.query_list(request, select,  isPO=False)

    async def post(self, request: Request, pk: int):
        """
        获取字典,table数据
        """
        param = DictFilter(pk)
        return await self.query_page(request, param)

    async def put(self, request: Request, pk: int):
        """
        编辑
        """
        async def before(oldPo: sysDictTypePO, po: sysDictTypePO, session: AsyncSession, request):
            exist = await db_tools.exist(session, sysDictTypePO.id != oldPo.id, sysDictTypePO.code.__eq__(po.code), column=sysDictTypePO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))

        return await self.edit(request, pk, sysDictTypePO, userId=self.getUserId(request), fun=before)

    async def delete(self, request: Request, pk: int):
        """
        删除
        """
        async def before(po: sysDictTypePO, session: AsyncSession):
            count = await db_tools.count(session, sysDictPO.dictTypeId == po.id, column=sysDictTypePO.id)
            if count > 0:
                return JSON_util.response(Result.fail(message=f"该'{po.name}'关联了字典不能删除！"))

        return await self.remove(request, pk, sysDictTypePO, beforeFun=before)




from sanic import Sanic, Blueprint, Request
from ..view_model.config import View, Views, ExistView, ConfigView
from co6co_sanic_ext.api import add_routes

config_api = Blueprint("config_API", url_prefix="/config")
add_routes(config_api, Views, View, ExistView, ConfigView)




from sanic import Request
from sqlalchemy.orm import joinedload
from sqlalchemy.sql import Select
from co6co.utils import log
from sqlalchemy.ext.asyncio import AsyncSession
from co6co_web_db.services.cacheManage import CacheManage
from . import getCurrentUserId, getSecret, generateCode


class BaseCache(CacheManage):
    userId: int
    request = Request

    def __init__(self, request: Request) -> None:
        # 微信认证中 userid可能为挂在上去
        self.userId = getCurrentUserId(request)
        self.request = request
        super().__init__(request.app)



from sanic import Request
from co6co_sanic_ext.model.res.result import Result
from ..base_view import AuthMethodView
from ...services import fileService


class batchDelView(AuthMethodView):
    routePath = "/batch/del"

    async def post(self, request: Request):
        paths = request.json.get("paths", [])
        try:
            flag = 0
            for path in paths:
                fileService.delFileOrFolder(path)
                flag += 1
            return self.response_json(Result.success(message="删除完成！"))
        except Exception as e:
            if flag > 0:
                return self.response_json(Result.fail(message="部分删除，因{}后面执行删除".format(e)))
            return self.response_json(Result.fail(message="删除失败：{}".format(e)))




from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, or_, and_, text as sqlText
from co6co_db_ext.db_utils import db_tools
from co6co_web_db.view_model import BaseMethodView, Request
from co6co_web_db.view_model import BaseMethodView
from ...model.enum import menu_type
from ...model.pos.right import menuPO, MenuRolePO, UserPO, UserGroupRolePO, UserRolePO
from ...model.filters.menu_filter import menu_filter
from ...services import getCurrentUserId
from ...services import getSecret, generatePageToken
from co6co_web_db.services import jwt_service
import uuid


class Session_View(BaseMethodView):
    routePath = "/"

    async def post(self, request: Request):
        """
        获取用户Session
        """
        session, _ = self.get_Session(request)
        # data = await jwt_service.createToken(getSecret(request), str(uuid.uuid4()),  session.expiry)
        return self.response_json(Result.success(data={"data":  str(uuid.uuid4()), "expiry": session.expiry}))




from sanic import Request
from sanic.response import file
from . import resource_baseView


class Res_Image_View(resource_baseView):
    routePath = "/img/<pk:int>"

    async def get(self, request: Request, pk: int):
        """
        显示图片
        """
        fullPath = await self.getLocalPathById(request, pk)
        return await file(fullPath, mime_type="image/jpeg")


class Res_Video_View(resource_baseView):
    routePath = "/video/<pk:int>"

    async def get(self, request: Request, pk: int):
        """
        显示视频文件
        """
        fullPath = await self.getLocalPathById(request, pk)
        return await file(fullPath, mime_type="image/jpeg")


class Res_thumbnail_View(resource_baseView):
    routePath = "/img/thumbnail/<pk:int>/<w:int>/<h:int>"

    async def get(self, request: Request, pk: int, w: int = 208, h: int = 117):
        """
        略缩图
        """
        fullPath = await self.getLocalPathById(request, pk)
        return await self.screenshot_image(fullPath, w, h)


class Res_Poster_View(resource_baseView):
    routePath = "/video/poster/<pk:int>/<w:int>/<h:int>"

    async def get(self, request: Request, pk: int, w: int = 208, h: int = 117):
        """
        视频截图
        视频第一帧作为 poster
        未使用可能需要
        """
        fullPath = await self.getLocalPathById(request, pk)
        return await self.screenshot(fullPath, w, h)





from co6co_sanic_ext.model.res.result import Result
from co6co_sanic_ext.utils import JSON_util
from ..model.enum import menu_type, menu_state, user_state
from ..view_model.aop.api_auth import authorized
from sanic import Sanic, Blueprint, Request
from ..view_model.menu_view import menu_tree_view, menu_view, menus_view, menu_batch_view

from co6co_sanic_ext.api import add_routes

menu_api = Blueprint("menu_API", url_prefix="/menu")
add_routes(menu_api, menus_view, menu_tree_view, menu_view, menu_batch_view)
'''
menu_api.add_route(menus_view.as_view(),"/",name=menus_view.__name__) 
menu_api.add_route(menu_tree_view.as_view(),"/tree",name=menu_tree_view.__name__) 
menu_api.add_route(menu_view.as_view(),"/<pk:int>",name=menu_view.__name__) 
'''
'''
代码放置在 view_model 可能会出现循环引用问题
'''


@menu_api.route("/status", methods=["GET", "POST"])
@authorized
async def getMenuStatus(request: Request):
    """
    菜单状态
    """
    states = menu_state.to_dict_list()
    return JSON_util.response(Result.success(data=states))


@menu_api.route("/category", methods=["GET", "POST"])
@authorized
async def getMenuCategory(request: Request):
    """
    菜单类别
    """
    states = menu_type.to_dict_list()
    return JSON_util.response(Result.success(data=states))



from sanic import Sanic, Blueprint, Request
from co6co_sanic_ext.api import add_routes
from ..view_model.user_group import user_groups_view, user_group_view, user_groups_tree_view, user_group_ass_view
from ..view_model.user_group import user_groups_sub_tree_view


userGroup_api = Blueprint("user_group_API", url_prefix="/userGroup")
add_routes(userGroup_api, user_groups_view, user_group_view, user_group_ass_view)
add_routes(userGroup_api, user_groups_tree_view, user_groups_sub_tree_view)

'''
userGroup_api.add_route(user_groups_view.as_view(),"/",name=user_groups_view.__name__) 
userGroup_api.add_route(user_groups_tree_view.as_view(),"/tree",name=user_groups_tree_view.__name__) 
userGroup_api.add_route(user_group_view.as_view(),"/<pk:int>",name=user_group_view.__name__) 
'''




from ...services.configCache import get_upload_path
from .utils import resize_image, screenshot, getTempFileName
from co6co .utils import log
from ..base_view import AuthMethodView
import json
import io
import os
from PIL import Image
from io import BytesIO
import requests
from sanic import Request
from sanic.response import text, raw, empty, file, file_stream, ResponseStream
from ...model.pos.resource import resourcePO
from co6co_db_ext.db_utils import QueryOneCallable
from sqlalchemy.sql import Select


class resource_baseView(AuthMethodView):
    async def getRersourcePath(self, request: Request, path):
        uploadRoot = await get_upload_path(request)
        fullPath = os.path.join(uploadRoot, path[1:])
        return os.path.abspath(fullPath)

    async def getLocalPathById(self, request: Request, pk: int) -> str:
        call = QueryOneCallable(self.get_db_session(request))
        data = await call(Select(resourcePO.url).filter(resourcePO.id == pk), isPO=False)
        if data != None:
            return await self.getRersourcePath(request, data["url"])
        return None

    async def getLocalPath(self, request: Request) -> str:
        path = ""
        for k, v in request.query_args:
            if k == "path":
                path = v
        if path.startswith("http"):
            return path
        return await self.getRersourcePath(request, path)

    async def screenshot(self, fullPath: str, w: int = 208, h: int = 117, isFile: bool = True):
        """
        视频截图
        视频第一帧作为 poster
        """
        if fullPath.startswith('http') or os.path.exists(fullPath):
            isFile = not fullPath.startswith('http')
            tempPath = await screenshot(fullPath, w, h, isFile=isFile, useBytes=True)
            if tempPath == None:
                return empty(status=404)
            return raw(tempPath,  status=200, headers=None,  content_type="image/jpeg")
        return empty(status=404)

    async def readHttpImage(self, url):
        data = requests.get(url)
        if data.status_code == 200:
            data = BytesIO(data.content)
            im = Image.open(data)
            return im
        return None

    async def readLocalImage(path):
        if os.path.exists(path):
            im = Image.open(path)
            return im
        return None

    async def screenshot_image(self, fullPath: str, w: int = 208, h: int = 117):
        """ 
        略缩图
        """
        im = None
        if fullPath.startswith('http'):
            im = await self.readHttpImage(fullPath)
        elif os.path.exists(fullPath):
            im = await self.readLocalImage(fullPath)
        if im != None:
            bytes = io.BytesIO()
            im.thumbnail((w, h))
            im.save(bytes, "PNG")
            return raw(bytes.getvalue(),  status=200, headers=None,  content_type="image/jpeg")
        return empty(status=404)




from sanic import Request
from sanic.response import file
from sanic.request.form import File
from sqlalchemy.orm import selectinload
from co6co.utils import log
from . import resource_baseView
from ...services.configCache import get_upload_path
from co6co_sanic_ext.model.res.result import Result
from ...model.pos.resource import resourcePO, userResourcePO
from ...model.enum import resource_category
from co6co.utils import hash, getDateFolder
import uuid
import os
from datetime import datetime
from co6co_db_ext.db_utils import DbCallable, db_tools

from sqlalchemy.sql import Select
from sqlalchemy.ext.asyncio import AsyncSession
from co6co_web_db.view_model import errorLog
from co6co.utils.tool_util import get_current_function_name


class FileResult:
    fullPath: str
    path: str
    hash: str
    name: str
    size: int

    def toPo(self, category: resource_category, subCategory: int = 0):
        po = resourcePO()
        po .uid = uuid.uuid4()
        po .category = category.val
        po .subCategory = subCategory
        po .hash = self.hash
        po .url = self.path
        po .metaName = self.name
        po .size = self.size
        po.createTime = datetime.now()
        return po


class Upload_View(resource_baseView):
    routePath = "/file"

    async def readFileSize(self, file: File):
        """
        获取文件大小
        """
        file_size = 0
        chunk_size = 1024
        while True:
            chunk = file.body.read(chunk_size)
            if not chunk:
                break
            file_size += len(chunk)
        file.body.seek(0)
        return file_size

    async def saveFile(self, request: Request):
        try:
            fullPath = None
            basePath = await get_upload_path(request)
            if "file" not in request.files:
                return self.response_json(Result.fail("No file part in the request"))
            file: File = request.files.get("file")

            size = len(file.body)  # await self. readFileSize(file)
            _, file_extension = os.path.splitext(file.name)
            fileName = getDateFolder(format='%Y-%m-%d-%H-%M-%S')
            fullPath, path = self.getFullPath(basePath, "{}{}".format(fileName, file_extension))
            await self.save_file(file, fullPath)
            result = FileResult()
            result.name = file.name
            result.hash = hash.file_sha256(fullPath)
            result.size = size
            result.fullPath = fullPath
            result.path = path

            return result
        except Exception as e:
            log.warn("保存文件失败：", fullPath, e)
            return self.response_json(Result.fail(message=e))

    async def saveDb(self, request: Request,  param: FileResult, category: resource_category) -> int:
        call = DbCallable(self.get_db_session(request))

        async def exec(session: AsyncSession):
            select = (Select(resourcePO).filter(resourcePO.hash.__eq__(param.hash)) .options(
                selectinload(resourcePO.userResourceList)
            ))

            dbPo: resourcePO = await db_tools.execForPo(session, select, remove_db_instance_state=False)
            userPo = userResourcePO()
            userPo.ownUserId = self.getUserId(request)
            userPo.createTime = datetime.now()
            userPo.name = param.name
            resourceId = None
            # 数据库中已经存在
            if dbPo != None:
                basePath = await get_upload_path(request)
                oldPath = os.path.join(basePath, dbPo.url[1:])
                if os.path.exists(oldPath) and os.path.exists(param.fullPath):
                    os.remove(param.fullPath)
                if not os.path.exists(oldPath):
                    dbPo.url = param.path

                dbPo.userResourceList.append(userPo)
                resourceId = dbPo.id
            else:
                po = param.toPo(category)
                po.userResourceList = [userPo]
                session.add(po)
                await session.flush()
                resourceId = po.id
            return resourceId
        return await call(exec)

    async def putFile(self, request: Request, category: resource_category):
        """
        上传文件
        """
        try:
            result = await self.saveFile(request)
            if type(result) != FileResult:
                return result
            result: FileResult = result
            resourceId = await self.saveDb(request, result, category)
            return self.response_json(Result.success(data={"resourceId": resourceId, "path": result.path}))
        except Exception as e:
            errorLog(request, self.__class__, get_current_function_name())
            return self.response_json(Result.fail(message="上传失败:{}".format(e)))

    async def put(self, request: Request):
        """
        上传文件
        """
        return await self.putFile(request, resource_category.file)


class Image_View(Upload_View):
    routePath = "/img"

    async def put(self, request: Request):
        """
        上传图片
        """
        return await self.putFile(request, resource_category.image)


class Video_View(Upload_View):
    routePath = "/video"

    async def put(self, request: Request):
        """
        上传视频
        """
        return await self.putFile(request, resource_category.video)




from co6co_web_db.view_model import BaseMethodView

from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result

from sqlalchemy.sql import Select,  Update
from co6co_db_ext.db_utils import db_tools
from co6co_web_db.services.jwt_service import createToken, setCurrentUser
from co6co.utils import log

from co6co_web_db.view_model import get_one
from ..model.pos.right import UserPO
from .aop.login_log import loginLog
from ..services import getSecret, generatePageToken
from ..model.enum import user_state
from datetime import datetime
from ..services.configCache import get_user_config


class login_view(BaseMethodView):
    routePath = "/login"

    @loginLog
    async def post(self, request: Request):
        """
        登录
        """
        try:
            where = UserPO()
            isLock = -1  # 1 锁定 0 解锁
            session = self.get_db_session(request)
            config = await get_user_config(request)
            failTimes = config.get("loginFail", 3)
            seconds = config.get("lockSeconds", 60)
            where.__dict__.update(request.json)
            status = [user_state.enabled.val, user_state.locked.val]
            select = Select(UserPO).filter(UserPO.userName.__eq__(where.userName), UserPO.state.in_(status))
            user: UserPO = await get_one(request, select)

            verifyCode = request.json.get("verifyCode", "")
            _, sessionDict = self.get_Session(request)
            memCode = sessionDict.pop("verifyCode")

            # // todo 为什么 应用 重启后 session 还在
            if verifyCode == "" or memCode != verifyCode:
                return JSON_util.response(Result.fail(message="验证码不能为空!"))
            if user != None:
                lockTime: datetime | None = user.lockTime
                if lockTime == None:
                    lockTime = datetime.now()
                lockTime = lockTime.timestamp()
                now = datetime.now().timestamp()
                waitingLogin: float = seconds-(now-lockTime)
                if user.state == user_state.locked.val and lockTime and waitingLogin > 0:
                    return JSON_util.response(Result.fail(message=f"账号已锁定，{round(waitingLogin, 3)}s后再试!"))

                if user.password == user.encrypt(where.password):
                    tokenData = await generatePageToken(getSecret(request), user, userOpenId=user.userGroupId)
                    # 让日志能获得用户信息
                    log.warn("用户状态", user.state)
                    if user.state == user_state.locked.val:
                        isLock = 0

                    await setCurrentUser(request, user.to_jwt_dict())
                    return JSON_util.response(Result.success(data=tokenData, message="登录成功"))
                else:
                    logingErrorCount = sessionDict.get("logingErrorCount", 0)
                    log.info("登录失败次数", logingErrorCount)
                    logingErrorCount += 1
                    sessionDict["logingErrorCount"] = logingErrorCount
                    if logingErrorCount == failTimes:
                        # 锁定
                        isLock = 1
                        sessionDict["logingErrorCount"] = 0
                        log.warn(f"用户登录失败次数达到{logingErrorCount}次，账号已锁定。")
                    return JSON_util.response(Result.fail(message="登录用户名或者密码不正确!"))
            else:
                log.warn(f"未找到用户名[{where.userName}],。")
                return JSON_util.response(Result.fail(message="登录用户名或者密码不正确!"))
        except Exception as e:
            log.err(f"登录失败:{e}", e)
            return JSON_util.response(Result.fail(message="登录失败!"))
        finally:
            if isLock != -1:
                updateSml = Update(UserPO).filter(UserPO.id.__eq__(user.id)).values(
                    {
                        UserPO.state: user_state.locked.val if isLock == 1 else user_state.enabled.val,
                        UserPO.lockTime: datetime.now() if isLock == 1 else None}
                )
                result = await db_tools.execSQL(session, updateSml)
                await session.commit()
            pass
